# -*- coding: utf-8 -*-
#  a2.py
#
# Copyright 2014-2019 Hiroshi Miura <miurahr@linux.com>
#

from __future__ import unicode_literals

from six import unichr


class a2 (object):
    _table_1 = ["\u3000", "\uff01", "\uff02", "\uff03", "\uff04", "\uff05", "\uff06",
                "\uff07", "\uff08", "\uff09", "\uff0a", "\uff0b", "\uff0c", "\uff0d",
                "\uff0e", "\uff0f",  # ！＂＃＄％＆＇（）＊＋，－．／
                "\uff10", "\uff11", "\uff12", "\uff13", "\uff14", "\uff15", "\uff16",
                "\uff17", "\uff18", "\uff19"  # ０...９
                "\uff1a", "\uff1b", "\uff1c", "\uff1d",
                "\uff1e", "\uff1f", "\uff20"]  # ：；＜＝＞？＠
    _table_2 = ["\uff3b", "\uff3c", "\uff3d", "\uff3e", "\uff3f", "\uff40"]  # ［＼］＾＿｀
    _table_3 = ["\uff5b", "\uff5c", "\uff5d", "\uff5e"]  # ｛｜｝～

    def __init__(self, mode):
        if mode == "E":
            self.convert = self.convert_E
        else:
            self.convert = self.convert_noop

    def isRegion(self, char):
        return 0x20 <= ord(char[0]) < 0x7f

    def _convert(self, text):
        c = ord(text[0])
        if 0x20 <= c < 0x41:
            return self._table_1[(c - 0x20)]
        elif 0x41 <= c < 0x5b:
            return unichr(0xff21 + c - 0x0041)  # u\0041A => u\ff21Ａ
        elif 0x5b <= c < 0x61:
            return self._table_2[(c - 0x5b)]
        elif 0x61 <= c < 0x7b:
            return unichr(0xff41 + c - 0x0061)  # u\0061a => u\ff41ａ
        elif 0x7b <= c < 0x7f:
            return self._table_3[(c - 0x7b)]
        else:
            return ""  # pragma: no cover

    def convert_E(self, text):
        t = self._convert(text)
        if len(t):
            return t, 1
        else:
            return "", 0

    def convert_noop(self, text):
        return text[0], 1



# -*- coding: utf-8 -*-
#  kanwa.py
#
# Copyright 2011-2019 Hiroshi Miura <miurahr@linux.com>

from marshal import loads
from zlib import decompress

import semidbm as dbm
from pkg_resources import resource_filename


# This class is Borg/Singleton
# It provides same results becase lookup from a static dictionary.
# There is no state rather dictionary dbm.
class kanwa (object):
    _shared_state = {}
    _kanwadict = None
    _jisyo_table = {}

    # FIXME: there is no invalidate mechanism for _jisyo_table data.
    #        It can lead a large memory consumption in long live process.
    #        but max size are limited to a size of a dictionary.

    def __new__(cls, *p, **k):
        self = object.__new__(cls, *p, **k)
        self.__dict__ = cls._shared_state
        return self

    def __init__(self):
        if self._kanwadict is None:
            dictpath = resource_filename(__name__, 'kanwadict3.db')  # FIXME: no hardcoded filename
            self._kanwadict = dbm.open(dictpath, 'r')

    def load(self, char):
        key = "%04x" % ord(char)
        if key in self._jisyo_table:
            return self._jisyo_table[key]
        else:
            try:
                self._jisyo_table[key] = loads(decompress(self._kanwadict[key]))
                return self._jisyo_table[key]
            except ValueError:
                return None  # pragma: no cover



# -*- coding: utf-8 -*-

import pykakasi


def test_issues60():
    TESTS = [
        (u"市立", u"しりつ")
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", None)
    kakasi.setMode("K", None)
    kakasi.setMode("J", "H")
    kakasi.setMode("s", False)
    kakasi.setMode("C", True)
    kakasi.setMode("E", None)
    kakasi.setMode("a", None)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_issues59():
    TESTS = [
        (u"じゃーん", u"じゃーん"),
        (u"ヷヸヹヺ", u"ヷヸヹヺ")
    ]
    kakasi = pykakasi.kakasi()
    kakasi.setMode("H", None)
    kakasi.setMode("K", "H")
    kakasi.setMode("J", None)
    kakasi.setMode("s", False)
    kakasi.setMode("C", True)
    kakasi.setMode("E", None)
    kakasi.setMode("a", None)
    converter = kakasi.getConverter()
    for case, result in TESTS:
        assert converter.do(case) == result


def test_kakasi_issues68():
    TESTS = [
        (u"", u""),
        (u"埇", u"よう")
    ]
    kks = pykakasi.kakasi()
    kks.setMode("J", "H")
    convert = kks.getConverter()
    for case, result in TESTS:
        assert convert.do(case) == result



#!/usr/bin/env python

import io
import os
import re
import shutil
from marshal import dumps
from zlib import compress

import semidbm as dbm
from setuptools import setup
from setuptools.command.build_py import build_py
from six import unichr
from six.moves import cPickle

package_name = "pykakasi"
root_dir = os.path.abspath(os.path.dirname(__file__))


class genkanwadict(object):
    records = {}

    def run(self, src, dst):
        with open(src, 'rb') as f:
            for line in f:
                self.parsekdict(line.decode("utf-8"))
            f.close()
        self.kanwaout(dst)

    # for itaiji and kana dict

    def mkdict(self, src, dst):
        max_key_len = 0
        dic = {}
        with open(src, "rb") as f:
            for line in f:
                line = line.decode("utf-8").strip()
                if line.startswith(';;'):  # skip comment
                    continue
                if re.match(r"^$", line):
                    continue
                try:
                    (v, k) = (re.sub(r'\\u([0-9a-fA-F]{4})',
                                     lambda x: unichr(int(x.group(1), 16)), line)).split(' ')
                    dic[k] = v
                    max_key_len = max(max_key_len, len(k))
                except ValueError:
                    raise Exception("Cannot process dictionary line: ", line)
        with open(dst, 'wb') as d:
            cPickle.dump((dic, max_key_len), d, protocol=2)

    # for kanwadict

    def parsekdict(self, line):
        line = line.strip()
        if line.startswith(';;'):  # skip comment
            return
        (yomi, kanji) = line.split(' ')
        if ord(yomi[-1:]) <= ord('z'):
            tail = yomi[-1:]
            yomi = yomi[:-1]
        else:
            tail = ''
        self.updaterec(kanji, yomi, tail)

    def updaterec(self, kanji, yomi, tail):
        key = "%04x" % ord(kanji[0])
        if key in self.records:
            if kanji in self.records[key]:
                rec = self.records[key][kanji]
                rec.append((yomi, tail))
                self.records[key].update({kanji: rec})
            else:
                self.records[key][kanji] = [(yomi, tail)]
        else:
            self.records[key] = {}
            self.records[key][kanji] = [(yomi, tail)]

    def kanwaout(self, out):
        dic = dbm.open(out, 'c')
        for (k, v) in self.records.items():
            dic[k] = compress(dumps(v))
        dic.close()


class MyBuild(build_py):

    def generate_dictionaries(self):
        DICTS = [
            ('itaijidict.utf8', 'itaijidict2.pickle'),
            ('hepburndict.utf8', 'hepburndict2.pickle'),
            ('kunreidict.utf8', 'kunreidict2.pickle'),
            ('passportdict.utf8', 'passportdict2.pickle'),
            ('hepburnhira.utf8', 'hepburnhira2.pickle'),
            ('kunreihira.utf8', 'kunreihira2.pickle'),
            ('passporthira.utf8', 'passporthira2.pickle')
        ]
        kanwa = genkanwadict()
        for (src_f, pkl_f) in DICTS:
            src = os.path.join('src', 'pykakasi', 'data', src_f)
            dst = os.path.join(self.build_lib, 'pykakasi', pkl_f)
            if (os.path.exists(dst)):
                os.unlink(dst)
            kanwa.mkdict(src, dst)
        src = os.path.join('src', 'pykakasi', 'data', 'kakasidict.utf8')
        dst = os.path.join(self.build_lib, 'pykakasi', 'kanwadict3.db')
        if (os.path.exists(dst)):
            shutil.rmtree(dst)
        kanwa.run(src, dst)

    def run(self):
        build_py.run(self)
        if not self.dry_run:
            self.generate_dictionaries()


def readme():
    with io.open(os.path.join(os.path.dirname(__file__), 'README.rst'), mode="r", encoding="UTF-8") as f:
        return f.read()


with open(os.path.join(root_dir, 'src', package_name, '__init__.py')) as f:
    init_text = f.read()
    version = re.search(r'__version__\s*=\s*[\'\"](.+?)[\'\"]', init_text).group(1)
    license = re.search(r'__license__\s*=\s*[\'\"](.+?)[\'\"]', init_text).group(1)
    author = re.search(r'__author__\s*=\s*[\'\"](.+?)[\'\"]', init_text).group(1)
    author_email = re.search(r'__author_email__\s*=\s*[\'\"](.+?)[\'\"]', init_text).group(1)
    url = re.search(r'__url__\s*=\s*[\'\"](.+?)[\'\"]', init_text).group(1)

assert version
assert license
assert author
assert author_email
assert url


setup(name=package_name,
      version=version,
      description='Python implementation of kakasi - kana kanji simple inversion library',
      url=url,
      license=license,
      long_description=readme(),
      author=author,
      author_email=author_email,
      package_dir={'pykakasi': 'src/pykakasi'},
      packages=[package_name],
      provides=[package_name],
      scripts=["bin/kakasi"],
      include_package_data=True,
      package_data={'src/pykakasi/data': ['*.utf8']},
      tests_require=['pytest', 'coverage'],
      setup_requires=['six', 'semidbm'],
      install_requires=['six', 'semidbm'],
      extras_require={'dev': ['pytest']},
      cmdclass={'build_py': MyBuild},
      classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Console',
          'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
          'Operating System :: MacOS :: MacOS X',
          'Operating System :: Microsoft :: Windows',
          'Operating System :: POSIX',
          'Operating System :: POSIX :: Linux',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          'Topic :: Software Development :: Libraries :: Python Modules']
      )



from .a2 import a2
from .h2 import H2
from .j2 import J2
from .k2 import K2
from .kakasi import kakasi
from .sym2 import sym2
from .wakati import wakati

__copyright__ = 'Copyright (C) 2010-2019 Hiroshi Miura'
__version__ = '0.95'
__license__ = 'GPLv3'
__author__ = 'Hiroshi Miura'
__author_email__ = 'miurahr@linux.com'
__url__ = 'http://github.com/miurahr/pykakasi'

__all__ = ["kakasi", "wakati", "exceptions", "J2", "K2", "H2", "sym2", "a2"]
